import { of, throwError } from 'rxjs';
import { Injectable, NgModule, APP_INITIALIZER } from '@angular/core';
import { HttpClient, HttpClientModule } from '@angular/common/http';
import { map, timeout, catchError, mergeMap } from 'rxjs/operators';
import { __extends } from 'tslib';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * DrupalConstants singleton design pattern of the required settings and user connection.
 */
// @dynamic
var  /**
 * DrupalConstants singleton design pattern of the required settings and user connection.
 */
// @dynamic
DrupalConstants = /** @class */ (function () {
    function DrupalConstants() {
        /**
         * handle offline error messages
         * @param err (optional) the error of the http request
         */
        this.handleOffline = (/**
         * @param {?=} err
         * @return {?}
         */
        function (err) {
            if (this.settings.allowOffline) {
                return of();
            }
            return throwError(err);
        });
    }
    Object.defineProperty(DrupalConstants, "Instance", {
        get: /**
         * @return {?}
         */
        function () {
            if (!this.instance) {
                this.instance = new DrupalConstants();
            }
            return this.instance;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DrupalConstants, "Settings", {
        /**
         * Handeling the case when user didn't implement the back-end settings and return the settings instance
         */
        get: /**
         * Handeling the case when user didn't implement the back-end settings and return the settings instance
         * @return {?}
         */
        function () {
            if (!this.Instance.settings) {
                throw new Error('ngx-drupal7-services: Application settings are not set, Please read README.MD file');
            }
            return this.Instance.settings;
        },
        /**
         * Set the settings instanse on application init.
         * you can also do that at the runtime, Thanks to Singleton design pattern <3 !
         */
        set: /**
         * Set the settings instanse on application init.
         * you can also do that at the runtime, Thanks to Singleton design pattern <3 !
         * @param {?} newSettings
         * @return {?}
         */
        function (newSettings) {
            this.Instance.settings = newSettings;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DrupalConstants, "backEndUrl", {
        /**
         * the full backend url for current user settings
         */
        get: /**
         * the full backend url for current user settings
         * @return {?}
         */
        function () {
            /** @type {?} */
            var settings = this.Settings;
            /** @type {?} */
            var url = settings.apiProtocol + '://' + settings.apiHost;
            return settings.apiPort ? url + ':' + settings.apiPort + '/' : url + '/';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DrupalConstants, "restUrl", {
        /**
         * Application back-end rest api url.
         */
        get: /**
         * Application back-end rest api url.
         * @return {?}
         */
        function () {
            /** @type {?} */
            var settings = this.Settings;
            return this.backEndUrl + settings.apiEndPoint + '/';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(DrupalConstants, "Connection", {
        /**
         * get the connection instance
         */
        get: /**
         * get the connection instance
         * @return {?}
         */
        function () {
            return this.Instance.connection;
        },
        /**
         * Updating connection instanse after login or rightaway after openning the connection
         */
        set: /**
         * Updating connection instanse after login or rightaway after openning the connection
         * @param {?} newConnection
         * @return {?}
         */
        function (newConnection) {
            this.Instance.connection = newConnection;
        },
        enumerable: true,
        configurable: true
    });
    return DrupalConstants;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * the main service is the basic http service of all other services "parent" that implements all the required request.
 * this service will add the headers automatically for each request it self
 */
var MainService = /** @class */ (function () {
    /**
     * the main constractor of this service will inject the required services dynamically.
     * @param http basic http service
     * @param cookieService ngx-cookie service provider to save the cookies
     * @see https://angular.io/guide/dependency-injection
     * @see https://www.npmjs.com/package/ngx-cookie
     */
    function MainService(httpClient) {
        this.httpClient = httpClient;
    }
    /**
     * structure a full entity for drupal request
     * @param entity the entity to be structured
     * @param ignoredFields fields to be ignored just like nid or uid or title
     * @param fieldLabels: the label for each field just like "{field_custom_field: 'value'}"
     * @param language language of the entity
     */
    /**
     * structure a full entity for drupal request
     * @param {?} entity the entity to be structured
     * @param {?} ignoredFields fields to be ignored just like nid or uid or title
     * @param {?=} fieldLabels
     * @param {?=} language language of the entity
     * @return {?}
     */
    MainService.prototype.structureEntity = /**
     * structure a full entity for drupal request
     * @param {?} entity the entity to be structured
     * @param {?} ignoredFields fields to be ignored just like nid or uid or title
     * @param {?=} fieldLabels
     * @param {?=} language language of the entity
     * @return {?}
     */
    function (entity, ignoredFields, fieldLabels, language) {
        var _this = this;
        Object.keys(entity).forEach((/**
         * @param {?} key
         * @param {?} index
         * @return {?}
         */
        function (key, index) {
            if (ignoredFields.indexOf(key) === -1) {
                /** @type {?} */
                var fieldLabel = void 0;
                if (fieldLabels[key]) {
                    fieldLabel = fieldLabels[key];
                }
                entity[key] = _this.structureField(entity[key], fieldLabel, language);
            }
        }));
        return entity;
    };
    /**
     * structure the field for drupal services request
     * @param value the field value
     * @param label field label name
     * @param language language of the field
     */
    /**
     * structure the field for drupal services request
     * @param {?} value the field value
     * @param {?=} label field label name
     * @param {?=} language language of the field
     * @return {?}
     */
    MainService.prototype.structureField = /**
     * structure the field for drupal services request
     * @param {?} value the field value
     * @param {?=} label field label name
     * @param {?=} language language of the field
     * @return {?}
     */
    function (value, label, language) {
        var _a, _b, _c, _d;
        if (label === void 0) { label = 'value'; }
        if (!language) {
            language = DrupalConstants.Settings.language;
        }
        /** @type {?} */
        var item = {};
        if (this.isArray(value)) {
            /** @type {?} */
            var field_array = [];
            for (var i = 0, l = value.length; i < l; i++) {
                item[label] = value[i];
                field_array.push(item);
            }
            return _a = {},
                _a[language] = field_array,
                _a;
        }
        if (value instanceof Date) {
            /** @type {?} */
            var obj = {
                value: {
                    date: value.getFullYear() + "-" + (value.getMonth() + 1) + "-" +
                        (value.getDate() + " " + value.getHours() + ":" + value.getMinutes() + ":" + value.getSeconds())
                }
            };
            return _b = {},
                _b[language] = [
                    obj
                ],
                _b;
        }
        // field value given with label(s) already built
        if (typeof value === 'object') {
            return _c = {},
                _c[language] = [
                    value
                ],
                _c;
        }
        item[label] = value;
        return _d = {},
            _d[language] = [
                item
            ],
            _d;
    };
    Object.defineProperty(MainService.prototype, "options", {
        /**
         * a getter to return the required headers for drupal
         * X-CSRF-Token - application token from services/session/connect
         * Content-Type - the type of the request content.
         * Accept - forcing drupal to return the response as a json object
         * @return object of the headers
         */
        get: /**
         * a getter to return the required headers for drupal
         * X-CSRF-Token - application token from services/session/connect
         * Content-Type - the type of the request content.
         * Accept - forcing drupal to return the response as a json object
         * @return {?} object of the headers
         */
        function () {
            /** @type {?} */
            var headers = {
                'Content-Type': 'application/json',
                'Accept': 'application/json',
            };
            /** @type {?} */
            var token = this.getSavedVariable('token');
            if (token) {
                headers['X-CSRF-Token'] = token;
            }
            headers['Authentication'] = this.getSavedVariable('session_name') + "=" + this.getSavedVariable('sessid');
            if (DrupalConstants.Settings.cookieHeader) {
                headers['Cookie'] = this.getSavedVariable('session_name') + "=" + this.getSavedVariable('sessid');
            }
            /** @type {?} */
            var options = {
                headers: headers,
                withCredentials: true,
                reportProgress: true,
            };
            return options;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * getting token from drupal services module
     * @return http text token response
     */
    /**
     * getting token from drupal services module
     * @protected
     * @return {?} http text token response
     */
    MainService.prototype.getToken = /**
     * getting token from drupal services module
     * @protected
     * @return {?} http text token response
     */
    function () {
        /** @type {?} */
        var options = this.options;
        options['responseType'] = 'text';
        return this.httpRequestWithConfig(this.httpClient.get(DrupalConstants.backEndUrl + "services/session/token", options)).pipe(map((/**
         * @param {?} res
         * @return {?}
         */
        function (res) {
            localStorage.setItem('token', res);
            return res;
        })));
    };
    /**
     * Saving drupal session and token in cookies using ngx-cookie service
     * @param connection drupal connection
     */
    /**
     * Saving drupal session and token in cookies using ngx-cookie service
     * @protected
     * @param {?} connection drupal connection
     * @return {?}
     */
    MainService.prototype.saveSessionToken = /**
     * Saving drupal session and token in cookies using ngx-cookie service
     * @protected
     * @param {?} connection drupal connection
     * @return {?}
     */
    function (connection) {
        if (!connection.token) {
            connection.token = this.getSavedVariable('token');
        }
        this.removeSession();
        DrupalConstants.Connection = connection;
        localStorage.setItem(connection.session_name, connection.sessid);
        localStorage.setItem('sessid', connection.sessid);
        localStorage.setItem('session_name', connection.session_name);
        localStorage.setItem('token', connection.token);
        if (connection.user && connection.user.timestamp) {
            localStorage.setItem('timestamp', connection.user.timestamp.toString());
        }
        else {
            localStorage.setItem('timestamp', Math.floor(Date.now()).toString());
        }
    };
    /**
     * building up the full url path for each resource and / or params
     * @param resource the entity resource param. ex: system/'connect', user/'login'
     * @return full request path after adding the entity type and resource param
     */
    /**
     * building up the full url path for each resource and / or params
     * @protected
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} full request path after adding the entity type and resource param
     */
    MainService.prototype.fullRequestURL = /**
     * building up the full url path for each resource and / or params
     * @protected
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} full request path after adding the entity type and resource param
     */
    function (resource) {
        /** @type {?} */
        var request_url = DrupalConstants.restUrl;
        if (this.entityType) {
            request_url += this.entityType;
        }
        if (resource) {
            if (request_url[request_url.length - 1] === '/') {
                request_url += resource;
            }
            else {
                request_url += '/' + resource;
            }
        }
        return request_url;
    };
    /**
     * adding http request configs: request timeout, error handler
     * @param httpObservableRequest the http Observable to request
     * @return Observable of the request after adding required configs
     */
    /**
     * adding http request configs: request timeout, error handler
     * @protected
     * @param {?} httpObservableRequest the http Observable to request
     * @return {?} Observable of the request after adding required configs
     */
    MainService.prototype.httpRequestWithConfig = /**
     * adding http request configs: request timeout, error handler
     * @protected
     * @param {?} httpObservableRequest the http Observable to request
     * @return {?} Observable of the request after adding required configs
     */
    function (httpObservableRequest) {
        return httpObservableRequest.pipe(timeout(DrupalConstants.Settings.requestTimeout), catchError((/**
         * @param {?} err
         * @return {?}
         */
        function (err) { return DrupalConstants.Instance.handleOffline(err); })));
    };
    /**
     * basic http get request with headers.
     * @param resource the entity resource param. ex: system/'connect', user/'login'
     * @return http json response
     */
    /**
     * basic http get request with headers.
     * @protected
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} http json response
     */
    MainService.prototype.get = /**
     * basic http get request with headers.
     * @protected
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} http json response
     */
    function (resource) {
        return this.httpRequestWithConfig(this.httpClient.get(this.fullRequestURL(resource), this.options));
    };
    /**
     * basic http post request with headers.
     * @param resource the entity resource param. ex: system/'connect', user/'login'
     * @param body the contenct of the request
     * @return http json response
     */
    /**
     * basic http post request with headers.
     * @protected
     * @param {?=} body the contenct of the request
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} http json response
     */
    MainService.prototype.post = /**
     * basic http post request with headers.
     * @protected
     * @param {?=} body the contenct of the request
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} http json response
     */
    function (body, resource) {
        if (body === void 0) { body = {}; }
        return this.httpRequestWithConfig(this.httpClient.post(this.fullRequestURL(resource), body, this.options));
    };
    /**
     * basic http put request with headers.
     * @param resource the entity resource param. ex: system/'connect', user/'login'
     * @param body the contenct of the request
     * @return http json response
     */
    /**
     * basic http put request with headers.
     * @protected
     * @param {?=} body the contenct of the request
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} http json response
     */
    MainService.prototype.put = /**
     * basic http put request with headers.
     * @protected
     * @param {?=} body the contenct of the request
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} http json response
     */
    function (body, resource) {
        if (body === void 0) { body = {}; }
        return this.httpRequestWithConfig(this.httpClient.put(this.fullRequestURL(resource), body, this.options));
    };
    /**
     * basic http delete request with headers.
     * @param resource the entity resource param. ex: system/'connect', user/'login'
     * @return http json response
     */
    /**
     * basic http delete request with headers.
     * @protected
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} http json response
     */
    MainService.prototype.delete = /**
     * basic http delete request with headers.
     * @protected
     * @param {?=} resource the entity resource param. ex: system/'connect', user/'login'
     * @return {?} http json response
     */
    function (resource) {
        return this.httpRequestWithConfig(this.httpClient.delete(this.fullRequestURL(resource), this.options));
    };
    /**
     * Clearing drupal session after logging out
     */
    /**
     * Clearing drupal session after logging out
     * @protected
     * @return {?}
     */
    MainService.prototype.removeSession = /**
     * Clearing drupal session after logging out
     * @protected
     * @return {?}
     */
    function () {
        localStorage.removeItem('token');
        localStorage.removeItem('timestamp');
        localStorage.removeItem('sessid');
        localStorage.removeItem('session_name');
        if (DrupalConstants.Connection && DrupalConstants.Connection.session_name) {
            localStorage.removeItem(DrupalConstants.Connection.session_name);
        }
    };
    /**
     * Checking the current connection if the connection is init and valid
     * @return if connection is valid
     */
    /**
     * Checking the current connection if the connection is init and valid
     * @protected
     * @return {?} if connection is valid
     */
    MainService.prototype.isConnected = /**
     * Checking the current connection if the connection is init and valid
     * @protected
     * @return {?} if connection is valid
     */
    function () {
        return this.getSavedVariable('token') &&
            this.getSavedVariable('sessid') &&
            this.getSavedVariable('session_name') &&
            !this.isConnectionExpired() ?
            true : false;
    };
    /**
     * Check if the drupal session is timedout.
     * @return true if the current date is less than the login date by 24 day "drupal default session timeout is 24 day".
     */
    /**
     * Check if the drupal session is timedout.
     * @protected
     * @return {?} true if the current date is less than the login date by 24 day "drupal default session timeout is 24 day".
     */
    MainService.prototype.isConnectionExpired = /**
     * Check if the drupal session is timedout.
     * @protected
     * @return {?} true if the current date is less than the login date by 24 day "drupal default session timeout is 24 day".
     */
    function () {
        /** @type {?} */
        var nowTS = Math.floor(Date.now());
        /** @type {?} */
        var sessionDays = 23;
        if (DrupalConstants.Settings.sessionDays) {
            sessionDays = DrupalConstants.Settings.sessionDays;
        }
        /** @type {?} */
        var dayMS = 86400000;
        // 1 day to ms
        /** @type {?} */
        var expirationTS = sessionDays * dayMS;
        return nowTS - +this.getSavedVariable('timestamp') < expirationTS;
    };
    /**
     * Serializin arguments as a string
     * @param options object of drupal parametars to serialize
     * @return string of parameters
     */
    /**
     * Serializin arguments as a string
     * @protected
     * @param {?} options object of drupal parametars to serialize
     * @return {?} string of parameters
     */
    MainService.prototype.getArgs = /**
     * Serializin arguments as a string
     * @protected
     * @param {?} options object of drupal parametars to serialize
     * @return {?} string of parameters
     */
    function (options) {
        var _this = this;
        if (!options) {
            return '';
        }
        /** @type {?} */
        var args = '?';
        Object.keys(options).forEach((/**
         * @param {?} key
         * @param {?} index
         * @return {?}
         */
        function (key, index) {
            args += _this.optionToString(key, options[key]);
        }));
        return args;
    };
    /**
     * Check if variable is array of objects
     * @param value array to check
     */
    /**
     * Check if variable is array of objects
     * @private
     * @param {?} value array to check
     * @return {?}
     */
    MainService.prototype.isArray = /**
     * Check if variable is array of objects
     * @private
     * @param {?} value array to check
     * @return {?}
     */
    function (value) {
        return Object.prototype.toString.call(value) === '[object Array]';
    };
    /**
     * retrive the variable from the connection object or if there is no connection yet it will return them from the cookies
     * this will will allow you to implement your custom connection storage, just like ionic.
     * because the cookies is not always available on webview apps, you may need to save the connection in sqllite
     * and restore them directly.
     * @param variableName the name of the saved variable
     */
    /**
     * retrive the variable from the connection object or if there is no connection yet it will return them from the cookies
     * this will will allow you to implement your custom connection storage, just like ionic.
     * because the cookies is not always available on webview apps, you may need to save the connection in sqllite
     * and restore them directly.
     * @private
     * @param {?} variableName the name of the saved variable
     * @return {?}
     */
    MainService.prototype.getSavedVariable = /**
     * retrive the variable from the connection object or if there is no connection yet it will return them from the cookies
     * this will will allow you to implement your custom connection storage, just like ionic.
     * because the cookies is not always available on webview apps, you may need to save the connection in sqllite
     * and restore them directly.
     * @private
     * @param {?} variableName the name of the saved variable
     * @return {?}
     */
    function (variableName) {
        if (DrupalConstants.Connection) {
            if (DrupalConstants.Connection[variableName]) {
                return DrupalConstants.Connection[variableName];
            }
            else if (DrupalConstants.Connection.user && DrupalConstants.Connection.user[variableName]) {
                return DrupalConstants.Connection.user[variableName];
            }
        }
        return localStorage.getItem(variableName);
    };
    /**
     * serializing eatch option
     * @param key option key
     * @param value option value
     * @return single option serilization
     */
    /**
     * serializing eatch option
     * @protected
     * @param {?} key option key
     * @param {?} value option value
     * @return {?} single option serilization
     */
    MainService.prototype.optionToString = /**
     * serializing eatch option
     * @protected
     * @param {?} key option key
     * @param {?} value option value
     * @return {?} single option serilization
     */
    function (key, value) {
        var _this = this;
        if (!value) {
            return '';
        }
        /** @type {?} */
        var str = '';
        if (value instanceof Array) {
            value.forEach((/**
             * @param {?} element
             * @param {?} index
             * @return {?}
             */
            function (element, index) {
                str += key + "[" + index + "]=" + element + "&";
            }));
        }
        else if (value instanceof Object) {
            Object.keys(value).forEach((/**
             * @param {?} element
             * @param {?} index
             * @return {?}
             */
            function (element, index) {
                if (value[element] instanceof Object) {
                    str += _this.serializeObject(value[element], key + "[" + element + "]");
                }
                else {
                    str += key + "[" + element + "]=" + value[element] + "&";
                }
            }));
        }
        else {
            str += key + "=" + value + "&";
        }
        return str;
    };
    /**
     * serializing the object keys
     * @param obj object to serialize
     */
    /**
     * serializing the object keys
     * @private
     * @param {?} obj object to serialize
     * @param {?} parentSerialized
     * @return {?}
     */
    MainService.prototype.serializeObject = /**
     * serializing the object keys
     * @private
     * @param {?} obj object to serialize
     * @param {?} parentSerialized
     * @return {?}
     */
    function (obj, parentSerialized) {
        var _this = this;
        /** @type {?} */
        var str = '';
        Object.keys(obj).forEach((/**
         * @param {?} key
         * @param {?} index
         * @return {?}
         */
        function (key, index) {
            /** @type {?} */
            var value = obj[key];
            if (value instanceof Object) {
                str += "" + _this.serializeObject(value, parentSerialized + "[" + key + "]");
            }
            else {
                str += parentSerialized + "[" + key + "]=" + value + "&";
            }
        }));
        return str;
    };
    MainService.decorators = [
        { type: Injectable }
    ];
    /** @nocollapse */
    MainService.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    return MainService;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * system service for drupal
 * @see BACK_END/admin/structure/services/list/END_POINT/resources for more details
 */
var SystemService = /** @class */ (function (_super) {
    __extends(SystemService, _super);
    function SystemService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'system';
        return _this;
    }
    /**
     * if there is already a token in the browser cookies and it is not expired it will return it
     * this will fetch a new token before trying to connect
     * @param refreshToken boolean to force the application to request new token
     * @return observable of the connect method
     * the subscription data is an object of SystemConnection interface
     */
    /**
     * if there is already a token in the browser cookies and it is not expired it will return it
     * this will fetch a new token before trying to connect
     * @param {?=} refreshToken boolean to force the application to request new token
     * @return {?} observable of the connect method
     * the subscription data is an object of SystemConnection interface
     */
    SystemService.prototype.connect = /**
     * if there is already a token in the browser cookies and it is not expired it will return it
     * this will fetch a new token before trying to connect
     * @param {?=} refreshToken boolean to force the application to request new token
     * @return {?} observable of the connect method
     * the subscription data is an object of SystemConnection interface
     */
    function (refreshToken) {
        var _this = this;
        if (refreshToken === void 0) { refreshToken = false; }
        if (this.isConnectionExpired()) {
            this.removeSession();
        }
        if (!DrupalConstants.Connection || !DrupalConstants.Connection.token || refreshToken) {
            return this.getToken().pipe(mergeMap((/**
             * @param {?} token
             * @return {?}
             */
            function (token) {
                return _this.post({}, 'connect').pipe(map((/**
                 * @param {?} connection
                 * @return {?}
                 */
                function (connection) {
                    connection.token = token;
                    _this.saveSessionToken(connection);
                    return connection;
                })));
            })));
        }
        return this.post({}, 'connect').pipe(map((/**
         * @param {?} connection
         * @return {?}
         */
        function (connection) {
            _this.saveSessionToken(connection);
            return connection;
        })));
    };
    /**
     * implement get_variable resource
     * @param variableName the name of the variable
     * @return the value of the variable
     */
    /**
     * implement get_variable resource
     * @param {?} variableName the name of the variable
     * @return {?} the value of the variable
     */
    SystemService.prototype.getVariable = /**
     * implement get_variable resource
     * @param {?} variableName the name of the variable
     * @return {?} the value of the variable
     */
    function (variableName) {
        return this.post({ name: variableName }, 'get_variable');
    };
    /**
     * implement set_variable resource
     * @param variableName the name of the variable
     * @param value the value to set for the variable
     * @return always null, take care of overriding old variables with same name
     */
    /**
     * implement set_variable resource
     * @param {?} variableName the name of the variable
     * @param {?} value the value to set for the variable
     * @return {?} always null, take care of overriding old variables with same name
     */
    SystemService.prototype.setVariable = /**
     * implement set_variable resource
     * @param {?} variableName the name of the variable
     * @param {?} value the value to set for the variable
     * @return {?} always null, take care of overriding old variables with same name
     */
    function (variableName, value) {
        /** @type {?} */
        var variable = {
            name: variableName,
            value: value
        };
        return this.post(variable, 'set_variable');
    };
    /**
     * implement del_variable resource
     * @param variableName variable name to delete
     * @return null if variable found or not.
     */
    /**
     * implement del_variable resource
     * @param {?} variableName variable name to delete
     * @return {?} null if variable found or not.
     */
    SystemService.prototype.delVariable = /**
     * implement del_variable resource
     * @param {?} variableName variable name to delete
     * @return {?} null if variable found or not.
     */
    function (variableName) {
        return this.post({ name: variableName }, 'del_variable');
    };
    SystemService.decorators = [
        { type: Injectable }
    ];
    return SystemService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * user service for drupal
 * @see BACK_END/admin/structure/services/list/END_POINT/resources for more details
 */
var UserService = /** @class */ (function (_super) {
    __extends(UserService, _super);
    function UserService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'user';
        return _this;
    }
    /**
     * implement retrive resource
     * @param uid user id to retrive
     * @return user object
     */
    /**
     * implement retrive resource
     * @param {?} uid user id to retrive
     * @return {?} user object
     */
    UserService.prototype.getUserById = /**
     * implement retrive resource
     * @param {?} uid user id to retrive
     * @return {?} user object
     */
    function (uid) {
        return this.get(uid);
    };
    /**
     * implement index resource
     * @return array of users
     * Please notice that the user must have the required permission to do this action
     */
    /**
     * implement index resource
     * @return {?} array of users
     * Please notice that the user must have the required permission to do this action
     */
    UserService.prototype.getUsersList = /**
     * implement index resource
     * @return {?} array of users
     * Please notice that the user must have the required permission to do this action
     */
    function () {
        return this.get();
    };
    /**
     * implement create resource
     * @param user user to create
     * @return created user details
     */
    /**
     * implement create resource
     * @param {?} user user to create
     * @return {?} created user details
     */
    UserService.prototype.createUser = /**
     * implement create resource
     * @param {?} user user to create
     * @return {?} created user details
     */
    function (user) {
        return this.post(user);
    };
    /**
     * implement update resource
     * @param user user object to update
     * @return updated user object
     */
    /**
     * implement update resource
     * @param {?} user user object to update
     * @return {?} updated user object
     */
    UserService.prototype.updateUser = /**
     * implement update resource
     * @param {?} user user object to update
     * @return {?} updated user object
     */
    function (user) {
        return this.put(user, user.uid);
    };
    /**
     * implement delete
     * @param uid user id to delete
     * @return array of boolean, if user deleted will be true, otherwise will be false
     */
    /**
     * implement delete
     * @param {?} uid user id to delete
     * @return {?} array of boolean, if user deleted will be true, otherwise will be false
     */
    UserService.prototype.deleteUser = /**
     * implement delete
     * @param {?} uid user id to delete
     * @return {?} array of boolean, if user deleted will be true, otherwise will be false
     */
    function (uid) {
        return this.delete(uid);
    };
    /**
     * implement login resource
     * @param user credentials to login "username and password"
     * @return system connection that contains the session details and the user object
     * notice that the user login version must be of version 1.0 at the resources settions at the back-end
     */
    /**
     * implement login resource
     * @param {?} user credentials to login "username and password"
     * @return {?} system connection that contains the session details and the user object
     * notice that the user login version must be of version 1.0 at the resources settions at the back-end
     */
    UserService.prototype.login = /**
     * implement login resource
     * @param {?} user credentials to login "username and password"
     * @return {?} system connection that contains the session details and the user object
     * notice that the user login version must be of version 1.0 at the resources settions at the back-end
     */
    function (user) {
        var _this = this;
        /** @type {?} */
        var observer = this.post(user, 'login');
        return observer.pipe(map((/**
         * @param {?} connection
         * @return {?}
         */
        function (connection) {
            _this.saveSessionToken(connection);
            return connection;
        })));
    };
    /**
     * implement logout resource
     * @return new token after logging out
     */
    /**
     * implement logout resource
     * @return {?} new token after logging out
     */
    UserService.prototype.logout = /**
     * implement logout resource
     * @return {?} new token after logging out
     */
    function () {
        var _this = this;
        /** @type {?} */
        var observer = this.post({}, 'logout');
        return observer.pipe(mergeMap((/**
         * @param {?} loggedOut
         * @return {?}
         */
        function (loggedOut) {
            if (loggedOut[0]) {
                _this.removeSession();
                return _this.getToken();
            }
        })));
    };
    /**
     * implement request_new_password resource
     * @param useranme the username of the user to request a new password
     * @return array of boolean, if the mail sent successfully will be true, otherwhise will be false.
     * if the server is not a mail server or if there is not mail server configuration this will always return false
     */
    /**
     * implement request_new_password resource
     * @param {?} useranme the username of the user to request a new password
     * @return {?} array of boolean, if the mail sent successfully will be true, otherwhise will be false.
     * if the server is not a mail server or if there is not mail server configuration this will always return false
     */
    UserService.prototype.requestNewPassword = /**
     * implement request_new_password resource
     * @param {?} useranme the username of the user to request a new password
     * @return {?} array of boolean, if the mail sent successfully will be true, otherwhise will be false.
     * if the server is not a mail server or if there is not mail server configuration this will always return false
     */
    function (useranme) {
        /** @type {?} */
        var user = {
            name: useranme
        };
        return this.post(user, 'request_new_password');
    };
    /**
     * implement user_pass_reset resource
     * @param passwordReset Object of the sent details to the email that contains the timestamp and the hashed password
     * @return response of the password reset request
     * this request will automatically logging the user in for only one time and will be expired after the first use.
     * the hashed password can be found at the drupal reset password email and you can customize it if you have token module installed
     */
    /**
     * implement user_pass_reset resource
     * @param {?} passwordReset Object of the sent details to the email that contains the timestamp and the hashed password
     * @return {?} response of the password reset request
     * this request will automatically logging the user in for only one time and will be expired after the first use.
     * the hashed password can be found at the drupal reset password email and you can customize it if you have token module installed
     */
    UserService.prototype.userPasswordReset = /**
     * implement user_pass_reset resource
     * @param {?} passwordReset Object of the sent details to the email that contains the timestamp and the hashed password
     * @return {?} response of the password reset request
     * this request will automatically logging the user in for only one time and will be expired after the first use.
     * the hashed password can be found at the drupal reset password email and you can customize it if you have token module installed
     */
    function (passwordReset) {
        return this.post(passwordReset, 'user_pass_reset');
    };
    /**
     * implement register resource
     * @param user object to register new account
     * some properties is required if it is set at the account settings inside the drupal admin config
     * @see http://BACKEND/admin/config/people/accounts
     * @return created user details
     */
    /**
     * implement register resource
     * @see http://BACKEND/admin/config/people/accounts
     * @param {?} user object to register new account
     * some properties is required if it is set at the account settings inside the drupal admin config
     * @return {?} created user details
     */
    UserService.prototype.registerAccount = /**
     * implement register resource
     * @see http://BACKEND/admin/config/people/accounts
     * @param {?} user object to register new account
     * some properties is required if it is set at the account settings inside the drupal admin config
     * @return {?} created user details
     */
    function (user) {
        return this.post(user, 'register');
    };
    /**
     * implement cancel resource
     * @param uid the user id to cancel
     * @return array of boolean, true if canceled, false otherwhise
     * Please notice the cancelation settings is managed at the backend
     * @see http://BACKEND/admin/config/people/accounts
     */
    /**
     * implement cancel resource
     * @see http://BACKEND/admin/config/people/accounts
     * @param {?} uid the user id to cancel
     * @return {?} array of boolean, true if canceled, false otherwhise
     * Please notice the cancelation settings is managed at the backend
     */
    UserService.prototype.cancelUser = /**
     * implement cancel resource
     * @see http://BACKEND/admin/config/people/accounts
     * @param {?} uid the user id to cancel
     * @return {?} array of boolean, true if canceled, false otherwhise
     * Please notice the cancelation settings is managed at the backend
     */
    function (uid) {
        return this.post({}, uid + "/cancel");
    };
    /**
     * implement password_reset resource
     * @param uid user id to reset his password
     * @return array of boolean, true if user password has been resetted, false otherwhise
     */
    /**
     * implement password_reset resource
     * @param {?} uid user id to reset his password
     * @return {?} array of boolean, true if user password has been resetted, false otherwhise
     */
    UserService.prototype.passwordReset = /**
     * implement password_reset resource
     * @param {?} uid user id to reset his password
     * @return {?} array of boolean, true if user password has been resetted, false otherwhise
     */
    function (uid) {
        return this.post({}, uid + "/password_reset");
    };
    /**
     * implement resend_welcome_email
     * @param uid user id to send the email for his account
     * @return array of boolean, if the mail sent successfully will be true, otherwhise will be false.
     * if the server is not a mail server or if there is not mail server configuration this will always return false
     */
    /**
     * implement resend_welcome_email
     * @param {?} uid user id to send the email for his account
     * @return {?} array of boolean, if the mail sent successfully will be true, otherwhise will be false.
     * if the server is not a mail server or if there is not mail server configuration this will always return false
     */
    UserService.prototype.resendWelcomeEmail = /**
     * implement resend_welcome_email
     * @param {?} uid user id to send the email for his account
     * @return {?} array of boolean, if the mail sent successfully will be true, otherwhise will be false.
     * if the server is not a mail server or if there is not mail server configuration this will always return false
     */
    function (uid) {
        return this.post({}, uid + "/resend_welcome_email");
    };
    /**
     * implement token
     */
    /**
     * implement token
     * @return {?}
     */
    UserService.prototype.token = /**
     * implement token
     * @return {?}
     */
    function () {
        return this.getToken();
    };
    UserService.decorators = [
        { type: Injectable }
    ];
    return UserService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * node service for drupal
 * @see BACK_END/admin/structure/services/list/END_POINT/resources for more details
 */
var NodeService = /** @class */ (function (_super) {
    __extends(NodeService, _super);
    function NodeService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'node';
        return _this;
    }
    /**
     * impelement index resource
     * @return array of nodes
     */
    /**
     * impelement index resource
     * @return {?} array of nodes
     */
    NodeService.prototype.getAllNodes = /**
     * impelement index resource
     * @return {?} array of nodes
     */
    function () {
        return this.get();
    };
    /**
     * implement retrive resource
     * @param nid node id to retrive
     * @return object of the node entity
     */
    /**
     * implement retrive resource
     * @param {?} nid node id to retrive
     * @return {?} object of the node entity
     */
    NodeService.prototype.getNodeById = /**
     * implement retrive resource
     * @param {?} nid node id to retrive
     * @return {?} object of the node entity
     */
    function (nid) {
        return this.get(nid);
    };
    /**
     * implement create resource
     * @param node node entity to create
     * @return created node details
     */
    /**
     * implement create resource
     * @param {?} node node entity to create
     * @return {?} created node details
     */
    NodeService.prototype.createNode = /**
     * implement create resource
     * @param {?} node node entity to create
     * @return {?} created node details
     */
    function (node) {
        return this.post(node);
    };
    /**
     * implement create resource
     * @param node node to create
     * @return created node details
     */
    /**
     * implement create resource
     * @param {?} node node to create
     * @return {?} created node details
     */
    NodeService.prototype.updateNode = /**
     * implement create resource
     * @param {?} node node to create
     * @return {?} created node details
     */
    function (node) {
        return this.put(node, node.nid);
    };
    /**
     * implement delete resource
     * @param nid node id to delete
     * @return array true if node deleted, otherwise return false array
     */
    /**
     * implement delete resource
     * @param {?} nid node id to delete
     * @return {?} array true if node deleted, otherwise return false array
     */
    NodeService.prototype.deleteNode = /**
     * implement delete resource
     * @param {?} nid node id to delete
     * @return {?} array true if node deleted, otherwise return false array
     */
    function (nid) {
        return this.delete(nid);
    };
    /**
     * implement files resource
     * @param nid node id
     * @return array of files that attached to that node
     */
    /**
     * implement files resource
     * @param {?} nid node id
     * @return {?} array of files that attached to that node
     */
    NodeService.prototype.files = /**
     * implement files resource
     * @param {?} nid node id
     * @return {?} array of files that attached to that node
     */
    function (nid) {
        return this.get(nid + "/files");
    };
    /**
    * implement comments resource
    * @param nid node id
    * @return array of comments that attached to that node
    */
    /**
     * implement comments resource
     * @param {?} nid node id
     * @return {?} array of comments that attached to that node
     */
    NodeService.prototype.comments = /**
     * implement comments resource
     * @param {?} nid node id
     * @return {?} array of comments that attached to that node
     */
    function (nid) {
        return this.get(nid + "/comments");
    };
    /**
    * implement attach_files resource
    * @param nid node id
    * @file object contains the field details and the file to upload
    * @return attached file
    */
    /**
     * implement attach_files resource
     * \@file object contains the field details and the file to upload
     * @param {?} nid node id
     * @param {?} file
     * @return {?} attached file
     */
    NodeService.prototype.attachFilesToNode = /**
     * implement attach_files resource
     * \@file object contains the field details and the file to upload
     * @param {?} nid node id
     * @param {?} file
     * @return {?} attached file
     */
    function (nid, file) {
        return this.post(file, nid + "/attach_file");
    };
    NodeService.decorators = [
        { type: Injectable }
    ];
    return NodeService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * file service for drupal
 * @see BACK_END/admin/structure/services/list/END_POINT/resources for more details
 */
var FileService = /** @class */ (function (_super) {
    __extends(FileService, _super);
    function FileService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'file';
        return _this;
    }
    /**
     * impelement index resource
     * @return array of files
     */
    /**
     * impelement index resource
     * @return {?} array of files
     */
    FileService.prototype.getAllFiles = /**
     * impelement index resource
     * @return {?} array of files
     */
    function () {
        return this.get();
    };
    /**
     * implement retrive resource
     * @param fid file id to retrive
     * @return object of the file entity
     */
    /**
     * implement retrive resource
     * @param {?} fid file id to retrive
     * @return {?} object of the file entity
     */
    FileService.prototype.getFileById = /**
     * implement retrive resource
     * @param {?} fid file id to retrive
     * @return {?} object of the file entity
     */
    function (fid) {
        return this.get(fid);
    };
    /**
     * implement create resource
     * @param file file entity to create
     * @return created file details
     */
    /**
     * implement create resource
     * @param {?} file file entity to create
     * @return {?} created file details
     */
    FileService.prototype.createFile = /**
     * implement create resource
     * @param {?} file file entity to create
     * @return {?} created file details
     */
    function (file) {
        return this.post(file);
    };
    /**
     * implement delete resource
     * @param fid file id to delete
     * @return boolean array if file deleted, otherwise return false
     * keep in mined that you can not delete a file if it is already referenced by a node
     */
    /**
     * implement delete resource
     * @param {?} fid file id to delete
     * @return {?} boolean array if file deleted, otherwise return false
     * keep in mined that you can not delete a file if it is already referenced by a node
     */
    FileService.prototype.deleteFile = /**
     * implement delete resource
     * @param {?} fid file id to delete
     * @return {?} boolean array if file deleted, otherwise return false
     * keep in mined that you can not delete a file if it is already referenced by a node
     */
    function (fid) {
        return this.delete(fid);
    };
    /**
     * creating raw file
     * this require rest support for form multi data
     */
    /**
     * creating raw file
     * this require rest support for form multi data
     * @return {?}
     */
    FileService.prototype.createRaw = /**
     * creating raw file
     * this require rest support for form multi data
     * @return {?}
     */
    function () {
        return this.post({}, 'create_raw');
    };
    FileService.decorators = [
        { type: Injectable }
    ];
    return FileService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * comment service for drupal
 * @see BACK_END/admin/structure/services/list/END_POINT/resources for more details
 */
var CommentService = /** @class */ (function (_super) {
    __extends(CommentService, _super);
    function CommentService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /**
         * entity type is comment
         */
        _this.entityType = 'comment';
        return _this;
    }
    /**
     * implement comment index resource
     * @return list of comments
     */
    /**
     * implement comment index resource
     * @return {?} list of comments
     */
    CommentService.prototype.getAllComments = /**
     * implement comment index resource
     * @return {?} list of comments
     */
    function () {
        return this.get();
    };
    /**
     * implement comment retrive method
     * @param cid comment id
     * @return drupal comment entity
     */
    /**
     * implement comment retrive method
     * @param {?} cid comment id
     * @return {?} drupal comment entity
     */
    CommentService.prototype.getCommentById = /**
     * implement comment retrive method
     * @param {?} cid comment id
     * @return {?} drupal comment entity
     */
    function (cid) {
        return this.get(cid);
    };
    /**
     * implement create resource
     * @param comment the comment to create
     * @return created comment details
     */
    /**
     * implement create resource
     * @param {?} comment the comment to create
     * @return {?} created comment details
     */
    CommentService.prototype.createComment = /**
     * implement create resource
     * @param {?} comment the comment to create
     * @return {?} created comment details
     */
    function (comment) {
        return this.post(comment);
    };
    /**
     * implement update resource
     * @param comment comment to update
     * @return array of comment id
     */
    /**
     * implement update resource
     * @param {?} comment comment to update
     * @return {?} array of comment id
     */
    CommentService.prototype.updateComment = /**
     * implement update resource
     * @param {?} comment comment to update
     * @return {?} array of comment id
     */
    function (comment) {
        return this.put(comment, comment.cid);
    };
    /**
     * implement delete resource
     * @param cid comment id to delete
     * @return array of boolean, if comment deleted the array will will contains true
     */
    /**
     * implement delete resource
     * @param {?} cid comment id to delete
     * @return {?} array of boolean, if comment deleted the array will will contains true
     */
    CommentService.prototype.deleteComment = /**
     * implement delete resource
     * @param {?} cid comment id to delete
     * @return {?} array of boolean, if comment deleted the array will will contains true
     */
    function (cid) {
        return this.delete(cid);
    };
    /**
     * implement countAll resource
     * @param nid host node id of the comments
     * @return number of comments on the node
     */
    /**
     * implement countAll resource
     * @param {?} nid host node id of the comments
     * @return {?} number of comments on the node
     */
    CommentService.prototype.countAllCommentsByNodeId = /**
     * implement countAll resource
     * @param {?} nid host node id of the comments
     * @return {?} number of comments on the node
     */
    function (nid) {
        return this.post({ nid: nid }, 'countAll');
    };
    /**
     * implement countNew resource.
     * @param nid host node id of the new comments
     * @return number of the new comments for the giving node id
     */
    /**
     * implement countNew resource.
     * @param {?} nid host node id of the new comments
     * @return {?} number of the new comments for the giving node id
     */
    CommentService.prototype.countNewCommentsByNodeId = /**
     * implement countNew resource.
     * @param {?} nid host node id of the new comments
     * @return {?} number of the new comments for the giving node id
     */
    function (nid) {
        return this.post({ nid: nid }, 'countNew');
    };
    CommentService.decorators = [
        { type: Injectable }
    ];
    return CommentService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * taxonomy term service for drupal
 * @see BACK_END/admin/structure/services/list/END_POINT/resources for more details
 */
var TaxonomyTermService = /** @class */ (function (_super) {
    __extends(TaxonomyTermService, _super);
    function TaxonomyTermService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'taxonomy_term';
        return _this;
    }
    /**
     * implement index resource
     * @return array of terms
     */
    /**
     * implement index resource
     * @return {?} array of terms
     */
    TaxonomyTermService.prototype.getAllTaxonomyTerms = /**
     * implement index resource
     * @return {?} array of terms
     */
    function () {
        return this.get();
    };
    /**
     * implement retrive resource
     * @param tid term id
     * @return term object
     */
    /**
     * implement retrive resource
     * @param {?} tid term id
     * @return {?} term object
     */
    TaxonomyTermService.prototype.getTaxonomyTermById = /**
     * implement retrive resource
     * @param {?} tid term id
     * @return {?} term object
     */
    function (tid) {
        return this.get(tid);
    };
    /**
     * implement create resource
     * @param taxonomyTerm the term to create
     * @return array number for the wight of the term
     */
    /**
     * implement create resource
     * @param {?} taxonomyTerm the term to create
     * @return {?} array number for the wight of the term
     */
    TaxonomyTermService.prototype.createTaxonomyTerm = /**
     * implement create resource
     * @param {?} taxonomyTerm the term to create
     * @return {?} array number for the wight of the term
     */
    function (taxonomyTerm) {
        return this.post(taxonomyTerm);
    };
    /**
     * implement update resource
     * @param taxonomyTerm term to update
     * @return array number for the wight of the term
     */
    /**
     * implement update resource
     * @param {?} taxonomyTerm term to update
     * @return {?} array number for the wight of the term
     */
    TaxonomyTermService.prototype.updateTaxonomyTerm = /**
     * implement update resource
     * @param {?} taxonomyTerm term to update
     * @return {?} array number for the wight of the term
     */
    function (taxonomyTerm) {
        return this.put(taxonomyTerm, taxonomyTerm.tid);
    };
    /**
     * implement delete resource
     * @param tid term id to delete
     * @return array of deleted term id
     */
    /**
     * implement delete resource
     * @param {?} tid term id to delete
     * @return {?} array of deleted term id
     */
    TaxonomyTermService.prototype.deleteTaxonomyTerm = /**
     * implement delete resource
     * @param {?} tid term id to delete
     * @return {?} array of deleted term id
     */
    function (tid) {
        return this.delete(tid);
    };
    /**
     * implement selectNodes
     * @param tid term id
     * @return array of nodes that referenced this term in one of the fields
     */
    /**
     * implement selectNodes
     * @param {?} tid term id
     * @return {?} array of nodes that referenced this term in one of the fields
     */
    TaxonomyTermService.prototype.getAllNodesForTerm = /**
     * implement selectNodes
     * @param {?} tid term id
     * @return {?} array of nodes that referenced this term in one of the fields
     */
    function (tid) {
        return this.post({ tid: tid }, 'selectNodes');
    };
    TaxonomyTermService.decorators = [
        { type: Injectable }
    ];
    return TaxonomyTermService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * taxonomy vocabulary service for drupal
 * @see BACK_END/admin/structure/services/list/END_POINT/resources for more details
 */
var TaxonomyVocabularyService = /** @class */ (function (_super) {
    __extends(TaxonomyVocabularyService, _super);
    function TaxonomyVocabularyService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'taxonomy_vocabulary';
        return _this;
    }
    /**
     * implement index resource
     * @return array of the available vocabularies
     */
    /**
     * implement index resource
     * @return {?} array of the available vocabularies
     */
    TaxonomyVocabularyService.prototype.getAllVocabularies = /**
     * implement index resource
     * @return {?} array of the available vocabularies
     */
    function () {
        return this.get();
    };
    /**
     * implement retrive resource
     * @param vid vocabulary id
     * @return vocabulary object
     */
    /**
     * implement retrive resource
     * @param {?} vid vocabulary id
     * @return {?} vocabulary object
     */
    TaxonomyVocabularyService.prototype.getTaxonomyVocabularyById = /**
     * implement retrive resource
     * @param {?} vid vocabulary id
     * @return {?} vocabulary object
     */
    function (vid) {
        return this.get(vid);
    };
    /**
     * implement create resource
     * @param taxonomyVocabulary the vocabulary to create
     * @return array of vocabulary id
     */
    /**
     * implement create resource
     * @param {?} taxonomyVocabulary the vocabulary to create
     * @return {?} array of vocabulary id
     */
    TaxonomyVocabularyService.prototype.createTaxonomyVocabulary = /**
     * implement create resource
     * @param {?} taxonomyVocabulary the vocabulary to create
     * @return {?} array of vocabulary id
     */
    function (taxonomyVocabulary) {
        return this.post(taxonomyVocabulary);
    };
    /**
     * implement update resource
     * @param taxonomyVocabulary the vocabulary to update
     * @return array of the current vocabulary whight
     */
    /**
     * implement update resource
     * @param {?} taxonomyVocabulary the vocabulary to update
     * @return {?} array of the current vocabulary whight
     */
    TaxonomyVocabularyService.prototype.updateTaxonomyVocabulary = /**
     * implement update resource
     * @param {?} taxonomyVocabulary the vocabulary to update
     * @return {?} array of the current vocabulary whight
     */
    function (taxonomyVocabulary) {
        return this.put(taxonomyVocabulary, taxonomyVocabulary.vid);
    };
    /**
     * implement delete resource
     * @param vid vocabulary id to delete
     * @return array of deleted vocabulary id
     */
    /**
     * implement delete resource
     * @param {?} vid vocabulary id to delete
     * @return {?} array of deleted vocabulary id
     */
    TaxonomyVocabularyService.prototype.deleteTaxonomyVocabulary = /**
     * implement delete resource
     * @param {?} vid vocabulary id to delete
     * @return {?} array of deleted vocabulary id
     */
    function (vid) {
        return this.delete(vid);
    };
    /**
     * implement retrieveByMachineName resource
     * @param vocabularyMachineName vocabulary machine name
     * @return vocabulary object
     */
    /**
     * implement retrieveByMachineName resource
     * @param {?} vocabularyMachineName vocabulary machine name
     * @return {?} vocabulary object
     */
    TaxonomyVocabularyService.prototype.getTaxonomyVocabularyByMachineName = /**
     * implement retrieveByMachineName resource
     * @param {?} vocabularyMachineName vocabulary machine name
     * @return {?} vocabulary object
     */
    function (vocabularyMachineName) {
        return this.post({ machine_name: vocabularyMachineName }, 'retrieveByMachineName');
    };
    /**
     * implement getTree resource
     * @param vid vocabulary id
     * @return a full list of taxonomy terms inside this vocabulary as a tree structure.
     */
    /**
     * implement getTree resource
     * @param {?} vid vocabulary id
     * @return {?} a full list of taxonomy terms inside this vocabulary as a tree structure.
     */
    TaxonomyVocabularyService.prototype.getTaxonomyVocabularyTree = /**
     * implement getTree resource
     * @param {?} vid vocabulary id
     * @return {?} a full list of taxonomy terms inside this vocabulary as a tree structure.
     */
    function (vid) {
        return this.post({ vid: vid }, 'getTree');
    };
    TaxonomyVocabularyService.decorators = [
        { type: Injectable }
    ];
    return TaxonomyVocabularyService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * view service for drupal using services_views module
 * @see https://www.drupal.org/project/services_views
 */
var ViewService = /** @class */ (function (_super) {
    __extends(ViewService, _super);
    function ViewService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'views';
        return _this;
    }
    /**
     * impelemnt retrive resource
     * @param viewMachineName view machine name
     * @param options parameters of filteration for this view
     * @return array of view rows OR view html code if the options format_output is 1
     */
    /**
     * impelemnt retrive resource
     * @param {?} viewMachineName view machine name
     * @param {?=} options parameters of filteration for this view
     * @return {?} array of view rows OR view html code if the options format_output is 1
     */
    ViewService.prototype.getView = /**
     * impelemnt retrive resource
     * @param {?} viewMachineName view machine name
     * @param {?=} options parameters of filteration for this view
     * @return {?} array of view rows OR view html code if the options format_output is 1
     */
    function (viewMachineName, options) {
        /** @type {?} */
        var args = this.getArgs(options);
        return this.get(viewMachineName + args);
    };
    ViewService.decorators = [
        { type: Injectable }
    ];
    return ViewService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * custom entity service for drupal using services_entity module
 * @see https://www.drupal.org/project/services_entity
 */
var EntityService = /** @class */ (function (_super) {
    __extends(EntityService, _super);
    function EntityService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        /**
         * the entity prefix for all the entities
         */
        _this.entityType = 'entity_';
        return _this;
    }
    /**
     * getting the full entity url for the entity
     * @param entityMachineName entity machine name, ex: node, user
     * @return string of full path to resource
     */
    /**
     * getting the full entity url for the entity
     * @protected
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @return {?} string of full path to resource
     */
    EntityService.prototype.fullRequestURL = /**
     * getting the full entity url for the entity
     * @protected
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @return {?} string of full path to resource
     */
    function (entityMachineName) {
        /** @type {?} */
        var request_url = DrupalConstants.restUrl;
        request_url += this.entityType + entityMachineName;
        return request_url;
    };
    /**
     * implement index resource
     * @param entityMachineName entity machine name, ex: node, user
     * @param options parameter options
     * @return array of the required entity type.
     * personally suggest to create an interface for each type you want to retrive,
     * you can use the models in this module as a perent interfaces.
     */
    /**
     * implement index resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?=} options parameter options
     * @return {?} array of the required entity type.
     * personally suggest to create an interface for each type you want to retrive,
     * you can use the models in this module as a perent interfaces.
     */
    EntityService.prototype.indexEntity = /**
     * implement index resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?=} options parameter options
     * @return {?} array of the required entity type.
     * personally suggest to create an interface for each type you want to retrive,
     * you can use the models in this module as a perent interfaces.
     */
    function (entityMachineName, options) {
        /** @type {?} */
        var args = this.getArgs(options);
        return this.get(entityMachineName + args);
    };
    /**
     * implement retrive resource
     * @param entityMachineName entity machine name, ex: node, user
     * @param selector the id of the entity
     * @param options parameter options
     * @return entity object
     */
    /**
     * implement retrive resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?} selector the id of the entity
     * @param {?=} options parameter options
     * @return {?} entity object
     */
    EntityService.prototype.retrieveEntity = /**
     * implement retrive resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?} selector the id of the entity
     * @param {?=} options parameter options
     * @return {?} entity object
     */
    function (entityMachineName, selector, options) {
        /** @type {?} */
        var args = this.getArgs(options);
        return this.get(entityMachineName + "/" + selector + args);
    };
    /**
     * implement delete resource
     * @param entityMachineName entity machine name, ex: node, user
     * @param selector the id of the entity
     * @return null if the entity is deleted,otherwise throws an error.
     */
    /**
     * implement delete resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?} selector the id of the entity
     * @return {?} null if the entity is deleted,otherwise throws an error.
     */
    EntityService.prototype.deleteEntity = /**
     * implement delete resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?} selector the id of the entity
     * @return {?} null if the entity is deleted,otherwise throws an error.
     */
    function (entityMachineName, selector) {
        return this.delete(entityMachineName + "/" + selector);
    };
    /**
     * implement create resource
     * @param entityMachineName entity machine name, ex: node, user
     * @param entity entity object to create
     * @return created entity
     */
    /**
     * implement create resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?} entity entity object to create
     * @return {?} created entity
     */
    EntityService.prototype.createEntity = /**
     * implement create resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?} entity entity object to create
     * @return {?} created entity
     */
    function (entityMachineName, entity) {
        return this.post(entity, entityMachineName);
    };
    /**
     * implement update resource
     * @param entityMachineName entity machine name, ex: node, user
     * @param entity entity to update
     * @param selector entity id to update and must match the id inside the entity object
     * @return updated entity
     */
    /**
     * implement update resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?} entity entity to update
     * @param {?} selector entity id to update and must match the id inside the entity object
     * @return {?} updated entity
     */
    EntityService.prototype.updateEntity = /**
     * implement update resource
     * @param {?} entityMachineName entity machine name, ex: node, user
     * @param {?} entity entity to update
     * @param {?} selector entity id to update and must match the id inside the entity object
     * @return {?} updated entity
     */
    function (entityMachineName, entity, selector) {
        return this.put(entity, entityMachineName + "/" + selector);
    };
    EntityService.decorators = [
        { type: Injectable }
    ];
    return EntityService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * menu service for drupal
 * @see https://www.drupal.org/project/services_menu for more details
 */
var MenuService = /** @class */ (function (_super) {
    __extends(MenuService, _super);
    function MenuService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'menu';
        return _this;
    }
    /**
     * implement menu retrive
     * @param menuMachineName menu machine name
     * @return menu witl all children links
     */
    /**
     * implement menu retrive
     * @param {?} menuMachineName menu machine name
     * @return {?} menu witl all children links
     */
    MenuService.prototype.getMenu = /**
     * implement menu retrive
     * @param {?} menuMachineName menu machine name
     * @return {?} menu witl all children links
     */
    function (menuMachineName) {
        return this.get(menuMachineName);
    };
    MenuService.decorators = [
        { type: Injectable }
    ];
    return MenuService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * facebook oauth service
 * @see https://www.drupal.org/project/services_fboauth
 * @see BACK_END/admin/structure/services/list/END_POINT/resources for more details
 */
var FacebookOAuthService = /** @class */ (function (_super) {
    __extends(FacebookOAuthService, _super);
    function FacebookOAuthService() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'fboauth';
        return _this;
    }
    /**
     * if there is already a token in the browser cookies and it is not expired it returns
     * this will fetch a new token before trying to connect
     * @param accessToken string of the recieved access token
     * @return observable of the connect method
     * the subscription data is an object of SystemConnection interface
     */
    /**
     * if there is already a token in the browser cookies and it is not expired it returns
     * this will fetch a new token before trying to connect
     * @param {?} accessToken string of the recieved access token
     * @return {?} observable of the connect method
     * the subscription data is an object of SystemConnection interface
     */
    FacebookOAuthService.prototype.connect = /**
     * if there is already a token in the browser cookies and it is not expired it returns
     * this will fetch a new token before trying to connect
     * @param {?} accessToken string of the recieved access token
     * @return {?} observable of the connect method
     * the subscription data is an object of SystemConnection interface
     */
    function (accessToken) {
        var _this = this;
        if (this.isConnectionExpired()) {
            this.removeSession();
        }
        /** @type {?} */
        var body = {
            access_token: accessToken
        };
        if (!DrupalConstants.Connection || !DrupalConstants.Connection.token) {
            return this.getToken().pipe(mergeMap((/**
             * @param {?} token
             * @return {?}
             */
            function (token) {
                return _this.post(body, 'connect').pipe(mergeMap((/**
                 * @param {?} connection
                 * @return {?}
                 */
                function (connection) {
                    return _this.getToken().pipe(map((/**
                     * @param {?} newToken
                     * @return {?}
                     */
                    function (newToken) {
                        connection.token = newToken;
                        _this.saveSessionToken(connection);
                        return connection;
                    })));
                })));
            })));
        }
        return this.post(body, 'connect').pipe(map((/**
         * @param {?} connection
         * @return {?}
         */
        function (connection) {
            _this.saveSessionToken(connection);
            return connection;
        })));
    };
    FacebookOAuthService.decorators = [
        { type: Injectable }
    ];
    return FacebookOAuthService;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * view service for drupal using push_notifications module
 * @see https://www.drupal.org/project/push_notifications
 */
var PushNotificationsSerivce = /** @class */ (function (_super) {
    __extends(PushNotificationsSerivce, _super);
    function PushNotificationsSerivce() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.entityType = 'push_notifications';
        return _this;
    }
    /**
     * create/register new device with a token
     * @param pushNotifications notification object to register
     */
    /**
     * create/register new device with a token
     * @param {?} pushNotifications notification object to register
     * @return {?}
     */
    PushNotificationsSerivce.prototype.create = /**
     * create/register new device with a token
     * @param {?} pushNotifications notification object to register
     * @return {?}
     */
    function (pushNotifications) {
        return this.post(pushNotifications);
    };
    /**
     * delete a subscription by device token
     * @param token token of the device to be deleted
     */
    /**
     * delete a subscription by device token
     * @param {?} token token of the device to be deleted
     * @return {?}
     */
    PushNotificationsSerivce.prototype.delete_push = /**
     * delete a subscription by device token
     * @param {?} token token of the device to be deleted
     * @return {?}
     */
    function (token) {
        return this.delete(token);
    };
    PushNotificationsSerivce.decorators = [
        { type: Injectable }
    ];
    return PushNotificationsSerivce;
}(MainService));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * implement APP_INITIALIZER
 * @see https://gillespie59.github.io/2016/12/04/angular2-code-before-rendering.html
 * @param {?} systemService system service to connect
 * @return {?}
 */
function init(systemService) {
    return (/**
     * @return {?}
     */
    function () {
        /** @type {?} */
        var connectionObservable = systemService.connect(true).toPromise();
        return connectionObservable;
    });
}
/**
 * main services module with providers
 * if you do not need to import all the services you need to make your own module and import the required providers only
 * ngx-cookie package is required
 * @see https://www.npmjs.com/package/ngx-cookie
 */
// @dynamic
var Drupal7ServicesModule = /** @class */ (function () {
    function Drupal7ServicesModule() {
    }
    Drupal7ServicesModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        HttpClientModule,
                    ],
                    providers: [
                        MainService,
                        SystemService,
                        UserService,
                        NodeService,
                        FileService,
                        CommentService,
                        TaxonomyTermService,
                        TaxonomyVocabularyService,
                        ViewService,
                        EntityService,
                        MenuService,
                        FacebookOAuthService,
                        PushNotificationsSerivce,
                        {
                            'provide': APP_INITIALIZER,
                            'useFactory': init,
                            'deps': [SystemService],
                            'multi': true
                        }
                    ],
                },] }
    ];
    return Drupal7ServicesModule;
}());

export { CommentService, Drupal7ServicesModule, DrupalConstants, EntityService, FacebookOAuthService, FileService, MainService, MenuService, NodeService, PushNotificationsSerivce, SystemService, TaxonomyTermService, TaxonomyVocabularyService, UserService, ViewService, init };
//# sourceMappingURL=ngx-drupal7-services.js.map
